<?php

namespace LakeDrops\DrupalDevelopmentEnvironment;

use LakeDrops\Component\Composer\BaseHandler;
use LakeDrops\Docker4Drupal\Handler as D4D;
use LakeDrops\DrupalEnvironment\Handler as DrupalEnvironment;
use Symfony\Component\Filesystem\Filesystem;

/**
 * Handler class to setup LakeDrops tool based Drupal development environments.
 *
 * @package LakeDrops\Drupal8ScaffoldDeveloper
 */
class Handler extends BaseHandler {

  /**
   * {@inheritdoc}
   */
  public function configId(): string {
    return 'lakedrops-dev';
  }

  /**
   * {@inheritdoc}
   */
  protected function configDefault(): array {
    $config = (new D4D($this->composer, $this->io))->getConfig();
    $domain = $config->readValue('projectname') . '.' . $config->readValue([
      'traefik',
      'domain',
    ]);
    if (!$config->readValue([
      'traefik',
      'usessl',
    ]) && $config->readValue(['traefik', 'port']) !== '80') {
      $domain .= ':' . $config->readValue(['traefik', 'port']);
    }
    return [
      'domain' => $domain,
      'coverage' => [],
      'symfony_deprecations_helper' => 'disabled',
    ];
  }

  /**
   * Post project create event to execute the scaffolding.
   */
  public function setupLakeDropsProject(): void {
    // We only do the fancy stuff for developers.
    if (!$this->isDevMode()) {
      return;
    }

    $this->init();

    $fs = new Filesystem();
    $installationManager = $this->composer->getInstallationManager();

    $drupalCorePackage = $this->getDrupalCorePackage();
    if ($drupalCorePackage === NULL) {
      return;
    }
    $corePath = $installationManager->getInstallPath($drupalCorePackage);

    // Directory where the root project is being created.
    $projectRoot = getcwd();
    // Directory where Drupal's index.php is located.
    $webRoot = dirname($corePath);
    // Directory where this plugin is being installed.
    $pluginRoot = $installationManager->getInstallPath($this->getPackage('lakedrops/drupal-development-environment'));

    // Create contrib/custom dirs for modules, profiles, recipes and themes.
    foreach ([
      $webRoot . '/modules',
      $webRoot . '/profiles',
      $webRoot . '/recipes',
      $webRoot . '/themes',
    ] as $dir) {
      foreach (['contrib', 'custom'] as $subdir) {
        $path = $dir . '/' . $subdir;
        if ($subdir === 'contrib') {
          $this->gitIgnore($path);
        }
        if (!$fs->exists($path)) {
          $fs->mkdir($path, 0775);
        }
        if (!$fs->exists($path . '/.gitkeep')) {
          $fs->touch($path . '/.gitkeep');
        }
      }
    }

    // Create and protect config, settings and files directories.
    foreach ([
      'config/default/sync',
      'tests',
    ] as $dir) {
      $path = $projectRoot . '/' . $dir;
      if (!$fs->exists($path)) {
        $fs->mkdir($path);
      }
    }

    // If available, also configure Drupal environment.
    if ($this->getPackage('lakedrops/drupal-environment')) {
      $handler = new DrupalEnvironment($this->composer, $this->io);
      $handler->setupLakeDropsProject();
    }

    // Tasks we only do the first time this is run.
    if (!$fs->exists($projectRoot . '/.git')) {

      // Remove template files.
      foreach (['.gitlab-ci.yml', 'README.md'] as $file) {
        $fs->remove($file);
      }

      // Initialize local git working copy.
      try {
        $this->git('init');
        $this->git('add *');
        $this->git('commit -m "Initial commit"');
      }
      catch (\Exception) {
        // We're ignoring this for now.
      }
    }

    // Also configure Docker4Drupal.
    $handler = new D4D($this->composer, $this->io);
    $handler->configureProject();

    // Copy Test files.
    foreach (['phpcs.files', 'phpstan.neon', 'phpunit.xml.dist'] as $template) {
      $rendered = $this->config->render($template, file_get_contents($pluginRoot . '/templates/tests/' . $template));
      file_put_contents('tests/' . $template, $rendered);
    }

    // Git Ignore test files.
    $this->gitIgnore('tests/phpcs.files');
    $this->gitIgnore('tests/phpstan.neon');
    $this->gitIgnore('tests/phpunit.xml.dist');
    $this->gitIgnore('tests/codesniffer_results.txt');
    $this->gitIgnore('tests/checkstyle.xml');
    $this->gitIgnore('tests/codesniffer_fixes.patch');
    $this->gitIgnore('tests/junit');
    $this->gitIgnore('tests/metrics');

    // Git Ignore output from simpletest.
    $this->gitIgnore('web/sites/simpletest');
  }

}
